#if os(iOS)
import UIKit

/// Alert with an indicator view at center.
public class ActivityViewController: UIViewController {
    
    /// `ActivityView` to use as view.
    let activityView = ActivityView()
    
    /// Initialize with given message.
    ///
    /// - Parameters:
    ///     - message: Message to be displayed in alert.
    ///
    /// - Returns: An alert with an indicator view at center and given message.
    public init(message: String) {
        super.init(nibName: nil, bundle: nil)
        modalTransitionStyle = .crossDissolve
        modalPresentationStyle = .overCurrentContext
        activityView.messageLabel.text = message
        view = activityView
    }
    
    /// init(coder:) is not implemented.
    public required init(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}

/// View used in `ActivityViewController` with an indicator view at center.
class ActivityView: UIView {
    
    /// Activity indicator.
    let activityIndicatorView = UIActivityIndicatorView(style: .whiteLarge)
    
    /// Alert view.
    let boundingBoxView = UIVisualEffectView(effect: UIBlurEffect(style: .dark))
    
    /// Message displayed.
    let messageLabel = UILabel(frame: CGRect.zero)
    
    
    /// Init this view.
    init() {
        super.init(frame: CGRect.zero)
        
        backgroundColor = .clear
        
        boundingBoxView.layer.cornerRadius = 12.0
        boundingBoxView.clipsToBounds = true
        
        activityIndicatorView.startAnimating()
        
        messageLabel.font = UIFont.boldSystemFont(ofSize: UIFont.labelFontSize)
        messageLabel.textColor = .white
        messageLabel.textAlignment = .center
        messageLabel.numberOfLines = 0
        
        addSubview(boundingBoxView)
        addSubview(activityIndicatorView)
        addSubview(messageLabel)
    }
    
    /// init(coder:) is not implemented.
    required init(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    /// Keep view's size.
    override func layoutSubviews() {
        super.layoutSubviews()
        
        boundingBoxView.frame.size.width = 160.0
        boundingBoxView.frame.size.height = 160.0
        boundingBoxView.frame.origin.x = ceil((bounds.width / 2.0) - (boundingBoxView.frame.width / 2.0))
        boundingBoxView.frame.origin.y = ceil((bounds.height / 2.0) - (boundingBoxView.frame.height / 2.0))
        
        activityIndicatorView.frame.origin.x = ceil((bounds.width / 2.0) - (activityIndicatorView.frame.width / 2.0))
        activityIndicatorView.frame.origin.y = ceil((bounds.height / 2.0) - (activityIndicatorView.frame.height / 2.0))
        
        let messageLabelSize = messageLabel.sizeThatFits(CGSize(width:160.0 - 20.0 * 2.0, height: CGFloat.greatestFiniteMagnitude))
        messageLabel.frame.size.width = messageLabelSize.width
        messageLabel.frame.size.height = messageLabelSize.height
        messageLabel.frame.origin.x = ceil((bounds.width / 2.0) - (messageLabel.frame.width / 2.0))
        messageLabel.frame.origin.y = ceil(activityIndicatorView.frame.origin.y + activityIndicatorView.frame.size.height + ((boundingBoxView.frame.height - activityIndicatorView.frame.height) / 4.0) - (messageLabel.frame.height / 2.0))
    }
}
#endif
